@@ -382,6 +382,8 @@ static CommandArray commands[] = {
     { "harvardcite",   CmdAuxHarvardCite, 0 },
     { "bibcite",       CmdBibCite,        0 },
     
+    {"usetikzlibrary",CmdTikzlib, 0},
+    
     {"", NULL, 0}
 };
 
@@ -800,6 +802,8 @@ static CommandArray params[] = {
 	{"LARGE", CmdFontSizeEnviron, 34},
 	{"huge", CmdFontSizeEnviron, 40},
 	{"Huge", CmdFontSizeEnviron, 50}, 
+
+    {"tikzpicture",CmdTikzPicture, 0},
 	
 	{"", NULL, 0}
 };                              /* end of list */
